//
//  ZPlatform.h
//  Part of Z-Way.C library
//
//  Created by Alex Skalozub on 1/6/12.
//  Based on Z-Way source code written by Christian Paetz and Poltorak Serguei
//
//  Copyright (c) 2012 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zway_platform_h
#define zway_platform_h

#include <stdio.h>

#ifdef _WINDOWS

#include <Windows.h>
#include <shlwapi.h>
#include <io.h>
#include <WinSock.h>

#pragma comment(lib, "shlwapi.lib")

typedef BYTE ZWBYTE;
typedef unsigned char ZWBOOL;

typedef TCHAR ZWCHAR;
typedef LPSTR ZWSTR;
typedef LPCSTR ZWCSTR;

typedef HANDLE ZWHANDLE;

#define copy_str(s) _tcsdup(s)
#define compare_str(s1, s2) _tcscmp(s1, s2)
#define compare_str_ci(s1, s2) _tcsicmp(s1, s2)
#define str_length(s) _tcslen(s)

#define ZSTR(s) TEXT(s)

typedef HANDLE ZWTHREAD;
typedef CRITICAL_SECTION ZWMUTEX;
typedef DWORD ZWMUTEXATTR;

#define current_thread() GetCurrentThread()

#else

#if defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
#define __sd_linux_use_directIO__ 1
#include <sys/io.h>
#endif

#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#ifndef __USE_UNIX98
  #define __USE_UNIX98 // to workaround PTHREAD_MUTEX_RECURSIVE and PTHREAD_MUTEX_RECURSIVE_NP
#endif
#include <pthread.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <termios.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>

#define SOCKET_ERROR -1
#define closesocket(x) close(x)
#define _sleep(x) usleep((x) * 1000)

#ifndef O_BINARY
#define O_BINARY 0
#endif

typedef unsigned char ZWBYTE;
typedef unsigned char ZWBOOL;

typedef char ZWCHAR;
typedef char* ZWSTR;
typedef const char* ZWCSTR;

typedef int ZWHANDLE;
typedef int SOCKET;

#define copy_str(s) strdup(s)
#define compare_str(s1, s2) strcmp(s1, s2)
#define compare_str_ci(s1, s2) stricmp(s1, s2)
#define str_length(s) strlen(s)

#define ZSTR(s) s

typedef pthread_t ZWTHREAD;
typedef pthread_mutex_t ZWMUTEX;
typedef pthread_mutexattr_t ZWMUTEXATTR;

#define current_thread() pthread_self()

#define TRUE 1
#define FALSE 0

#endif

#define SET_BIT(bitmask, index) ((ZWBYTE*)(bitmask))[(index) / 8] |= (ZWBYTE)(1 << ((index) % 8))
#define CLR_BIT(bitmask, index) ((ZWBYTE*)(bitmask))[(index) / 8] &= (ZWBYTE)(~(1 << ((index) % 8)))
#define TEST_BIT(bitmask, index) (((ZWBYTE*)(bitmask))[(index) / 8] & (ZWBYTE)(1 << ((index) % 8)))

ZWSTR sys_last_err_string();

#endif
